@@ -23,8 +23,8 @@ Tools for logging and tracing.
 # we abuse conventions to give a consistent interface 
 # pylint: disable-msg=C0103
 
-from lepl.monitor import ExposedMonitor
-from lepl.support import CircularFifo
+from lepl.monitor import ActiveMonitor, ValueMonitor
+from lepl.support import CircularFifo, LogMixin, sample, format, str
 
 
 def TraceResults(enabled=False):
@@ -33,12 +33,13 @@ def TraceResults(enabled=False):
     that records the flow of control during parsing.  It can be controlled by 
     `Trace()`.
 
-    This is a helper function that "escapes" the main class via a function
-    to simplify configuration.
+    This is a factory that "escapes" the main class via a function to simplify 
+    configuration.
     '''
     return lambda: _TraceResults(enabled)
 
-class _TraceResults(ExposedMonitor):
+
+class _TraceResults(ActiveMonitor, ValueMonitor, LogMixin):
     '''
     A basic logger (implemented as a monitor - `MonitorInterface`)
     that records the flow of control during parsing.  It can be controlled by 
@@ -66,7 +67,7 @@ class _TraceResults(ExposedMonitor):
         '''
         if self.enabled > 0:
             self.generator = generator
-            self.action = 'next({0})'.format(generator.describe)
+            self.action = format('next({0})', generator.describe)
     
     def after_next(self, value):
         '''
@@ -82,11 +83,13 @@ class _TraceResults(ExposedMonitor):
         if self.enabled > 0:
             self.generator = generator
             if type(value) is StopIteration:
-                self.action = ' stop -> {0}({1!r})'.format(
-                            generator.matcher.describe, generator.stream)
+                self.action = format(' stop -> {0}({1!r})',
+                                     generator.matcher.describe, 
+                                     generator.stream)
             else:
-                self.action = '{2!r} -> {0}({1!r})'.format(
-                            generator.matcher.describe, generator.stream, value)
+                self.action = format('{2!r} -> {0}({1!r})',
+                                     generator.matcher.describe, 
+                                     generator.stream, value)
     
     def after_throw(self, value):
         '''
@@ -101,8 +104,8 @@ class _TraceResults(ExposedMonitor):
         '''
         if self.enabled > 0:
             self.generator = generator
-            self.action = '{1!r} -> {0}'.format(
-                            generator.matcher.describe, value)
+            self.action = format('{1!r} -> {0}',
+                                 generator.matcher.describe, value)
     
     def after_send(self, value):
         '''
@@ -119,54 +122,52 @@ class _TraceResults(ExposedMonitor):
             if type(value) is StopIteration:
                 self._log_done(self.fmt_done())
             else:
-                self._log_error(value, self.fmt_result(value))
+                self._log_error(self.fmt_result(value))
         
     def fmt_result(self, value):
         '''
         Provide a standard format for the results.
         '''
         (stream, depth, locn) = self.fmt_stream() 
-        return '{0:05d} {1:11s} {2} ({3:04d}) {4:03d} {5:>60s} -> {6!r}' \
-                .format(self.epoch, 
-                        stream,
-                        locn,
-                        depth,
-                        self.depth,
-                        self.action,
-                        value)
+        return format('{0:05d} {1!r:11s} {2} ({3:04d}) {4:03d} '
+                      '{5:>60s} -> {6!r}',
+                      self.epoch, 
+                      stream,
+                      locn,
+                      depth,
+                      self.depth,
+                      self.action,
+                      value)
                 
     def fmt_done(self):
         '''
         Provide a standard format for failure.
         '''
         (stream, depth, locn) = self.fmt_stream() 
-        return '{0:05d} {1:11s} {2} ({3:04d}) {4:03d} {5:>60s} -> stop' \
-                .format(self.epoch, 
-                        stream,
-                        locn,
-                        depth,
-                        self.depth,
-                        self.action)
+        return format('{0:05d} {1!r:11s} {2} ({3:04d}) {4:03d} '
+                      '{5:>60s} -> stop',
+                      self.epoch, 
+                      stream,
+                      locn,
+                      depth,
+                      self.depth,
+                      self.action)
                 
     def fmt_stream(self):
         '''
         Provide a standard format for location.
         '''
         try:
-            (lineno, offset, depth, text, source) = \
-                    self.generator.stream.location()
-            stream = str(self.generator.stream)
+            (lineno, offset, depth, _text, _source) = \
+                    self.generator.stream.location
             if lineno < 0:
                 locn = '  eof  '
             else:
-                locn = '{0:3d}.{1:<3d}'.format(lineno, offset)
-        except AttributeError: # no .location() above
+                locn = format('{0:3d}.{1:<3d}', lineno, offset)
+        except AttributeError: # no .location above
             depth = -len(self.generator.stream)
-            if len(self.generator.stream) > 10:
-                stream = repr(self.generator.stream[0:7]) + '...'
-            else:
-                stream = repr(self.generator.stream)
             locn = '<unknown>'
+        stream = sample('', str(self.generator.stream), 9)
         return (stream, depth, locn)
         
     def yield_(self, value):
@@ -182,19 +183,19 @@ class _TraceResults(ExposedMonitor):
         '''
         if self.enabled > 0:
             if type(value) is StopIteration:
-                self._info(self.fmt_final_result('raise {0!r}'.format(value)))
+                self._info(self.fmt_final_result(format('raise {0!r}', value)))
             else:
-                self._warn(self.fmt_final_result('raise {0!r}'.format(value)))
+                self._warn(self.fmt_final_result(format('raise {0!r}', value)))
         
     def fmt_final_result(self, value):
         '''
         Provide a standard format for the result.
         '''
-        return '{0:05d}                            {1:03d} {2} {3}' \
-                .format(self.epoch,
-                        self.depth,
-                        ' ' * 63,
-                        value)
+        return format('{0:05d}                            {1:03d} {2} {3}',
+                      self.epoch,
+                      self.depth,
+                      ' ' * 63,
+                      value)
 
     def _log_result(self, value, text):
         '''
@@ -202,7 +203,7 @@ class _TraceResults(ExposedMonitor):
         '''
         (self._info if type(value) is tuple else self._debug)(text)    
 
-    def _log_error(self, value, text):
+    def _log_error(self, text):
         '''
         Record an error.
         '''
@@ -258,7 +259,7 @@ class _RecordDeepest(_TraceResults):
         if type(value) is tuple:
             self.record(True, text)
 
-    def _log_error(self, value, text):
+    def _log_error(self, text):
         '''
         Modify `TraceResults` to record the data.
         '''
@@ -324,11 +325,11 @@ class _RecordDeepest(_TraceResults):
         '''
         Format the result.
         '''
-        return \
-            '\nUp to {0} matches before and including longest match:\n{1}\n' \
-            'Up to {2} failures following longest match:\n{3}\n' \
-            'Up to {4} successful matches following longest match:\n{5}\n' \
-            .format(self.n_before, '\n'.join(self._before),
-                    self.n_done_after, '\n'.join(self._done_after),
-                    self.n_results_after, '\n'.join(self._results_after))
+        return format(
+            '\nUp to {0} matches before and including longest match:\n{1}\n'
+            'Up to {2} failures following longest match:\n{3}\n'
+            'Up to {4} successful matches following longest match:\n{5}\n',
+            self.n_before, '\n'.join(self._before),
+            self.n_done_after, '\n'.join(self._done_after),
+            self.n_results_after, '\n'.join(self._results_after))
         
